@@ -102,15 +102,16 @@ from [Temporal's hybrid grammar].
 // support a span of time of about 52 hours or so.)
 
 use crate::{
-    error::{err, Error, ErrorContext},
+    error::{fmt::offset::Error as E, Error, ErrorContext},
     fmt::{
+        buffer::ArrayBuffer,
         temporal::{PiecesNumericOffset, PiecesOffset},
-        util::{parse_temporal_fraction, FractionalFormatter},
+        util::parse_temporal_fraction,
         Parsed,
     },
     tz::Offset,
     util::{
-        escape, parse,
+        parse,
         rangeint::{ri8, RFrom},
         t::{self, C},
     },
@@ -237,13 +238,7 @@ impl Numeric {
             if part_nanoseconds >= C(500_000_000) {
                 seconds = seconds
                     .try_checked_add("offset-seconds", C(1))
-                    .with_context(|| {
-                        err!(
-                            "due to precision loss, UTC offset '{}' is \
-                             rounded to a value that is out of bounds",
-                            self,
-                        )
-                    })?;
+                    .context(E::PrecisionLoss)?;
             }
         }
         Ok(Offset::from_seconds_ranged(seconds * self.sign))
@@ -254,27 +249,35 @@ impl Numeric {
 // `Offset` fails.
 impl core::fmt::Display for Numeric {
     fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
-        if self.sign == C(-1) {
-            write!(f, "-")?;
-        } else {
-            write!(f, "+")?;
-        }
-        write!(f, "{:02}", self.hours)?;
+        let mut buf = ArrayBuffer::<19>::default();
+        let mut bbuf = buf.as_borrowed();
+
+        bbuf.write_ascii_char(if self.sign == C(-1) { b'-' } else { b'+' });
+        bbuf.write_int_pad2(self.hours.get().unsigned_abs());
         if let Some(minutes) = self.minutes {
-            write!(f, ":{:02}", minutes)?;
+            bbuf.write_ascii_char(b':');
+            bbuf.write_int_pad2(minutes.get().unsigned_abs());
         }
         if let Some(seconds) = self.seconds {
-            write!(f, ":{:02}", seconds)?;
+            if self.minutes.is_none() {
+                bbuf.write_str(":00");
+            }
+            bbuf.write_ascii_char(b':');
+            bbuf.write_int_pad2(seconds.get().unsigned_abs());
         }
         if let Some(nanos) = self.nanoseconds {
-            static FMT: FractionalFormatter = FractionalFormatter::new();
-            write!(
-                f,
-                ".{}",
-                FMT.format(i32::from(nanos).unsigned_abs()).as_str()
-            )?;
+            if nanos != C(0) {
+                if self.minutes.is_none() {
+                    bbuf.write_str(":00");
+                }
+                if self.seconds.is_none() {
+                    bbuf.write_str(":00");
+                }
+                bbuf.write_ascii_char(b'.');
+                bbuf.write_fraction(None, nanos.get().unsigned_abs());
+            }
         }
-        Ok(())
+        f.write_str(bbuf.filled())
     }
 }
 
@@ -413,18 +416,14 @@ impl Parser {
         mut input: &'i [u8],
     ) -> Result<Parsed<'i, ParsedOffset>, Error> {
         if input.is_empty() {
-            return Err(err!("expected UTC offset, but found end of input"));
+            return Err(Error::from(E::EndOfInput));
         }
 
         if input[0] == b'Z' || input[0] == b'z' {
             if !self.zulu {
-                return Err(err!(
-                    "found {z:?} in {original:?} where a numeric UTC offset \
-                     was expected (this context does not permit \
-                     the Zulu offset)",
-                    z = escape::Byte(input[0]),
-                    original = escape::Bytes(input),
-                ));
+                return Err(Error::from(E::UnexpectedLetterOffsetNoZulu(
+                    input[0],
+                )));
             }
             input = &input[1..];
             let value = ParsedOffset { kind: ParsedOffsetKind::Zulu };
@@ -464,40 +463,24 @@ impl Parser {
         &self,
         input: &'i [u8],
     ) -> Result<Parsed<'i, Numeric>, Error> {
-        let original = escape::Bytes(input);
-
         // Parse sign component.
         let Parsed { value: sign, input } =
-            self.parse_sign(input).with_context(|| {
-                err!("failed to parse sign in UTC numeric offset {original:?}")
-            })?;
+            self.parse_sign(input).context(E::InvalidSign)?;
 
         // Parse hours component.
         let Parsed { value: hours, input } =
-            self.parse_hours(input).with_context(|| {
-                err!(
-                    "failed to parse hours in UTC numeric offset {original:?}"
-                )
-            })?;
+            self.parse_hours(input).context(E::InvalidHours)?;
         let extended = match self.colon {
             Colon::Optional => input.starts_with(b":"),
             Colon::Required => {
                 if !input.is_empty() && !input.starts_with(b":") {
-                    return Err(err!(
-                        "parsed hour component of time zone offset from \
-                         {original:?}, but could not find required colon \
-                         separator",
-                    ));
+                    return Err(Error::from(E::NoColonAfterHours));
                 }
                 true
             }
             Colon::Absent => {
                 if !input.is_empty() && input.starts_with(b":") {
-                    return Err(err!(
-                        "parsed hour component of time zone offset from \
-                         {original:?}, but found colon after hours which \
-                         is not allowed",
-                    ));
+                    return Err(Error::from(E::ColonAfterHours));
                 }
                 false
             }
@@ -513,32 +496,22 @@ impl Parser {
         };
 
         // Parse optional separator after hours.
-        let Parsed { value: has_minutes, input } =
-            self.parse_separator(input, extended).with_context(|| {
-                err!(
-                    "failed to parse separator after hours in \
-                     UTC numeric offset {original:?}"
-                )
-            })?;
+        let Parsed { value: has_minutes, input } = self
+            .parse_separator(input, extended)
+            .context(E::SeparatorAfterHours)?;
         if !has_minutes {
-            if self.require_minute || (self.subminute && self.require_second) {
-                return Err(err!(
-                    "parsed hour component of time zone offset from \
-                     {original:?}, but could not find required minute \
-                     component",
-                ));
-            }
-            return Ok(Parsed { value: numeric, input });
+            return if self.require_minute
+                || (self.subminute && self.require_second)
+            {
+                Err(Error::from(E::MissingMinuteAfterHour))
+            } else {
+                Ok(Parsed { value: numeric, input })
+            };
         }
 
         // Parse minutes component.
         let Parsed { value: minutes, input } =
-            self.parse_minutes(input).with_context(|| {
-                err!(
-                    "failed to parse minutes in UTC numeric offset \
-                     {original:?}"
-                )
-            })?;
+            self.parse_minutes(input).context(E::InvalidMinutes)?;
         numeric.minutes = Some(minutes);
 
         // If subminute resolution is not supported, then we're done here.
@@ -549,65 +522,42 @@ impl Parser {
             // more precision than is supported. So we return an error here.
             // If this winds up being problematic, we can make this error
             // configurable or remove it altogether (unfortunate).
-            if input.get(0).map_or(false, |&b| b == b':') {
-                return Err(err!(
-                    "subminute precision for UTC numeric offset {original:?} \
-                     is not enabled in this context (must provide only \
-                     integral minutes)",
-                ));
-            }
-            return Ok(Parsed { value: numeric, input });
+            return if input.get(0).map_or(false, |&b| b == b':') {
+                Err(Error::from(E::SubminutePrecisionNotEnabled))
+            } else {
+                Ok(Parsed { value: numeric, input })
+            };
         }
 
         // Parse optional separator after minutes.
-        let Parsed { value: has_seconds, input } =
-            self.parse_separator(input, extended).with_context(|| {
-                err!(
-                    "failed to parse separator after minutes in \
-                     UTC numeric offset {original:?}"
-                )
-            })?;
+        let Parsed { value: has_seconds, input } = self
+            .parse_separator(input, extended)
+            .context(E::SeparatorAfterMinutes)?;
         if !has_seconds {
-            if self.require_second {
-                return Err(err!(
-                    "parsed hour and minute components of time zone offset \
-                     from {original:?}, but could not find required second \
-                     component",
-                ));
-            }
-            return Ok(Parsed { value: numeric, input });
+            return if self.require_second {
+                Err(Error::from(E::MissingSecondAfterMinute))
+            } else {
+                Ok(Parsed { value: numeric, input })
+            };
         }
 
         // Parse seconds component.
         let Parsed { value: seconds, input } =
-            self.parse_seconds(input).with_context(|| {
-                err!(
-                    "failed to parse seconds in UTC numeric offset \
-                     {original:?}"
-                )
-            })?;
+            self.parse_seconds(input).context(E::InvalidSeconds)?;
         numeric.seconds = Some(seconds);
 
         // If subsecond resolution is not supported, then we're done here.
         if !self.subsecond {
             if input.get(0).map_or(false, |&b| b == b'.' || b == b',') {
-                return Err(err!(
-                    "subsecond precision for UTC numeric offset {original:?} \
-                     is not enabled in this context (must provide only \
-                     integral minutes or seconds)",
-                ));
+                return Err(Error::from(E::SubsecondPrecisionNotEnabled));
             }
             return Ok(Parsed { value: numeric, input });
         }
 
         // Parse an optional fractional component.
         let Parsed { value: nanoseconds, input } =
-            parse_temporal_fraction(input).with_context(|| {
-                err!(
-                    "failed to parse fractional nanoseconds in \
-                     UTC numeric offset {original:?}",
-                )
-            })?;
+            parse_temporal_fraction(input)
+                .context(E::InvalidSecondsFractional)?;
         // OK because `parse_temporal_fraction` guarantees `0..=999_999_999`.
         numeric.nanoseconds =
             nanoseconds.map(|n| t::SubsecNanosecond::new(n).unwrap());
@@ -619,19 +569,13 @@ impl Parser {
         &self,
         input: &'i [u8],
     ) -> Result<Parsed<'i, t::Sign>, Error> {
-        let sign = input.get(0).copied().ok_or_else(|| {
-            err!("expected UTC numeric offset, but found end of input")
-        })?;
+        let sign = input.get(0).copied().ok_or(E::EndOfInputNumeric)?;
         let sign = if sign == b'+' {
             t::Sign::N::<1>()
         } else if sign == b'-' {
             t::Sign::N::<-1>()
         } else {
-            return Err(err!(
-                "expected '+' or '-' sign at start of UTC numeric offset, \
-                 but found {found:?} instead",
-                found = escape::Byte(sign),
-            ));
+            return Err(Error::from(E::InvalidSignPlusOrMinus));
         };
         Ok(Parsed { value: sign, input: &input[1..] })
     }
@@ -641,22 +585,16 @@ impl Parser {
         &self,
         input: &'i [u8],
     ) -> Result<Parsed<'i, ParsedOffsetHours>, Error> {
-        let (hours, input) = parse::split(input, 2).ok_or_else(|| {
-            err!("expected two digit hour after sign, but found end of input",)
-        })?;
-        let hours = parse::i64(hours).with_context(|| {
-            err!(
-                "failed to parse {hours:?} as hours (a two digit integer)",
-                hours = escape::Bytes(hours),
-            )
-        })?;
+        let (hours, input) =
+            parse::split(input, 2).ok_or(E::EndOfInputHour)?;
+        let hours = parse::i64(hours).context(E::ParseHours)?;
         // Note that we support a slightly bigger range of offsets than
         // Temporal. Temporal seems to support only up to 23 hours, but
         // we go up to 25 hours. This is done to support POSIX time zone
         // strings, which also require 25 hours (plus the maximal minute/second
         // components).
         let hours = ParsedOffsetHours::try_new("hours", hours)
-            .context("offset hours are not valid")?;
+            .context(E::RangeHours)?;
         Ok(Parsed { value: hours, input })
     }
 
@@ -665,20 +603,11 @@ impl Parser {
         &self,
         input: &'i [u8],
     ) -> Result<Parsed<'i, ParsedOffsetMinutes>, Error> {
-        let (minutes, input) = parse::split(input, 2).ok_or_else(|| {
-            err!(
-                "expected two digit minute after hours, \
-                 but found end of input",
-            )
-        })?;
-        let minutes = parse::i64(minutes).with_context(|| {
-            err!(
-                "failed to parse {minutes:?} as minutes (a two digit integer)",
-                minutes = escape::Bytes(minutes),
-            )
-        })?;
+        let (minutes, input) =
+            parse::split(input, 2).ok_or(E::EndOfInputMinute)?;
+        let minutes = parse::i64(minutes).context(E::ParseMinutes)?;
         let minutes = ParsedOffsetMinutes::try_new("minutes", minutes)
-            .context("minutes are not valid")?;
+            .context(E::RangeMinutes)?;
         Ok(Parsed { value: minutes, input })
     }
 
@@ -687,20 +616,11 @@ impl Parser {
         &self,
         input: &'i [u8],
     ) -> Result<Parsed<'i, ParsedOffsetSeconds>, Error> {
-        let (seconds, input) = parse::split(input, 2).ok_or_else(|| {
-            err!(
-                "expected two digit second after hours, \
-                 but found end of input",
-            )
-        })?;
-        let seconds = parse::i64(seconds).with_context(|| {
-            err!(
-                "failed to parse {seconds:?} as seconds (a two digit integer)",
-                seconds = escape::Bytes(seconds),
-            )
-        })?;
+        let (seconds, input) =
+            parse::split(input, 2).ok_or(E::EndOfInputSecond)?;
+        let seconds = parse::i64(seconds).context(E::ParseSeconds)?;
         let seconds = ParsedOffsetSeconds::try_new("seconds", seconds)
-            .context("time zone offset seconds are not valid")?;
+            .context(E::RangeSeconds)?;
         Ok(Parsed { value: seconds, input })
     }
 
@@ -941,7 +861,7 @@ mod tests {
     fn err_numeric_empty() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"").unwrap_err(),
-            @r###"failed to parse sign in UTC numeric offset "": expected UTC numeric offset, but found end of input"###,
+            @"failed to parse sign in UTC numeric offset: expected UTC numeric offset, but found end of input",
         );
     }
 
@@ -950,7 +870,7 @@ mod tests {
     fn err_numeric_notsign() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"*").unwrap_err(),
-            @r###"failed to parse sign in UTC numeric offset "*": expected '+' or '-' sign at start of UTC numeric offset, but found "*" instead"###,
+            @"failed to parse sign in UTC numeric offset: expected `+` or `-` sign at start of UTC numeric offset",
         );
     }
 
@@ -959,7 +879,7 @@ mod tests {
     fn err_numeric_hours_too_short() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"+a").unwrap_err(),
-            @r###"failed to parse hours in UTC numeric offset "+a": expected two digit hour after sign, but found end of input"###,
+            @"failed to parse hours in UTC numeric offset: expected two digit hour after sign, but found end of input",
         );
     }
 
@@ -968,7 +888,7 @@ mod tests {
     fn err_numeric_hours_invalid_digits() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"+ab").unwrap_err(),
-            @r###"failed to parse hours in UTC numeric offset "+ab": failed to parse "ab" as hours (a two digit integer): invalid digit, expected 0-9 but got a"###,
+            @"failed to parse hours in UTC numeric offset: failed to parse hours (requires a two digit integer): invalid digit, expected 0-9 but got a",
         );
     }
 
@@ -977,7 +897,7 @@ mod tests {
     fn err_numeric_hours_out_of_range() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-26").unwrap_err(),
-            @r###"failed to parse hours in UTC numeric offset "-26": offset hours are not valid: parameter 'hours' with value 26 is not in the required range of 0..=25"###,
+            @"failed to parse hours in UTC numeric offset: hour in time zone offset is out of range: parameter 'hours' with value 26 is not in the required range of 0..=25",
         );
     }
 
@@ -986,7 +906,7 @@ mod tests {
     fn err_numeric_minutes_too_short() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"+05:a").unwrap_err(),
-            @r###"failed to parse minutes in UTC numeric offset "+05:a": expected two digit minute after hours, but found end of input"###,
+            @"failed to parse minutes in UTC numeric offset: expected two digit minute after hours, but found end of input",
         );
     }
 
@@ -995,7 +915,7 @@ mod tests {
     fn err_numeric_minutes_invalid_digits() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"+05:ab").unwrap_err(),
-            @r###"failed to parse minutes in UTC numeric offset "+05:ab": failed to parse "ab" as minutes (a two digit integer): invalid digit, expected 0-9 but got a"###,
+            @"failed to parse minutes in UTC numeric offset: failed to parse minutes (requires a two digit integer): invalid digit, expected 0-9 but got a",
         );
     }
 
@@ -1004,7 +924,7 @@ mod tests {
     fn err_numeric_minutes_out_of_range() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-05:60").unwrap_err(),
-            @r###"failed to parse minutes in UTC numeric offset "-05:60": minutes are not valid: parameter 'minutes' with value 60 is not in the required range of 0..=59"###,
+            @"failed to parse minutes in UTC numeric offset: minute in time zone offset is out of range: parameter 'minutes' with value 60 is not in the required range of 0..=59",
         );
     }
 
@@ -1013,7 +933,7 @@ mod tests {
     fn err_numeric_seconds_too_short() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"+05:30:a").unwrap_err(),
-            @r###"failed to parse seconds in UTC numeric offset "+05:30:a": expected two digit second after hours, but found end of input"###,
+            @"failed to parse seconds in UTC numeric offset: expected two digit second after minutes, but found end of input",
         );
     }
 
@@ -1022,7 +942,7 @@ mod tests {
     fn err_numeric_seconds_invalid_digits() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"+05:30:ab").unwrap_err(),
-            @r###"failed to parse seconds in UTC numeric offset "+05:30:ab": failed to parse "ab" as seconds (a two digit integer): invalid digit, expected 0-9 but got a"###,
+            @"failed to parse seconds in UTC numeric offset: failed to parse seconds (requires a two digit integer): invalid digit, expected 0-9 but got a",
         );
     }
 
@@ -1031,7 +951,7 @@ mod tests {
     fn err_numeric_seconds_out_of_range() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-05:30:60").unwrap_err(),
-            @r###"failed to parse seconds in UTC numeric offset "-05:30:60": time zone offset seconds are not valid: parameter 'seconds' with value 60 is not in the required range of 0..=59"###,
+            @"failed to parse seconds in UTC numeric offset: second in time zone offset is out of range: parameter 'seconds' with value 60 is not in the required range of 0..=59",
         );
     }
 
@@ -1041,31 +961,31 @@ mod tests {
     fn err_numeric_fraction_non_empty() {
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-05:30:44.").unwrap_err(),
-            @r###"failed to parse fractional nanoseconds in UTC numeric offset "-05:30:44.": found decimal after seconds component, but did not find any decimal digits after decimal"###,
+            @"failed to parse fractional seconds in UTC numeric offset: found decimal after seconds component, but did not find any digits after decimal",
         );
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-05:30:44,").unwrap_err(),
-            @r###"failed to parse fractional nanoseconds in UTC numeric offset "-05:30:44,": found decimal after seconds component, but did not find any decimal digits after decimal"###,
+            @"failed to parse fractional seconds in UTC numeric offset: found decimal after seconds component, but did not find any digits after decimal",
         );
 
         // Instead of end-of-string, add invalid digit.
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-05:30:44.a").unwrap_err(),
-            @r###"failed to parse fractional nanoseconds in UTC numeric offset "-05:30:44.a": found decimal after seconds component, but did not find any decimal digits after decimal"###,
+            @"failed to parse fractional seconds in UTC numeric offset: found decimal after seconds component, but did not find any digits after decimal",
         );
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-05:30:44,a").unwrap_err(),
-            @r###"failed to parse fractional nanoseconds in UTC numeric offset "-05:30:44,a": found decimal after seconds component, but did not find any decimal digits after decimal"###,
+            @"failed to parse fractional seconds in UTC numeric offset: found decimal after seconds component, but did not find any digits after decimal",
         );
 
         // And also test basic format.
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-053044.a").unwrap_err(),
-            @r###"failed to parse fractional nanoseconds in UTC numeric offset "-053044.a": found decimal after seconds component, but did not find any decimal digits after decimal"###,
+            @"failed to parse fractional seconds in UTC numeric offset: found decimal after seconds component, but did not find any digits after decimal",
         );
         insta::assert_snapshot!(
             Parser::new().parse_numeric(b"-053044,a").unwrap_err(),
-            @r###"failed to parse fractional nanoseconds in UTC numeric offset "-053044,a": found decimal after seconds component, but did not find any decimal digits after decimal"###,
+            @"failed to parse fractional seconds in UTC numeric offset: found decimal after seconds component, but did not find any digits after decimal",
         );
     }
 
@@ -1076,7 +996,7 @@ mod tests {
     fn err_numeric_subminute_disabled_but_desired() {
         insta::assert_snapshot!(
             Parser::new().subminute(false).parse_numeric(b"-05:59:32").unwrap_err(),
-            @r###"subminute precision for UTC numeric offset "-05:59:32" is not enabled in this context (must provide only integral minutes)"###,
+            @"subminute precision for UTC numeric offset is not enabled in this context (must provide only integral minutes)",
         );
     }
 
@@ -1086,11 +1006,11 @@ mod tests {
     fn err_zulu_disabled_but_desired() {
         insta::assert_snapshot!(
             Parser::new().zulu(false).parse(b"Z").unwrap_err(),
-            @r###"found "Z" in "Z" where a numeric UTC offset was expected (this context does not permit the Zulu offset)"###,
+            @"found `Z` where a numeric UTC offset was expected (this context does not permit the Zulu offset)",
         );
         insta::assert_snapshot!(
             Parser::new().zulu(false).parse(b"z").unwrap_err(),
-            @r###"found "z" in "z" where a numeric UTC offset was expected (this context does not permit the Zulu offset)"###,
+            @"found `z` where a numeric UTC offset was expected (this context does not permit the Zulu offset)",
         );
     }
 
@@ -1118,7 +1038,7 @@ mod tests {
         };
         insta::assert_snapshot!(
             numeric.to_offset().unwrap_err(),
-            @"due to precision loss, UTC offset '+25:59:59.5' is rounded to a value that is out of bounds: parameter 'offset-seconds' with value 1 is not in the required range of -93599..=93599",
+            @"due to precision loss, offset is rounded to a value that is out of bounds: parameter 'offset-seconds' with value 1 is not in the required range of -93599..=93599",
         );
     }
 
@@ -1143,7 +1063,7 @@ mod tests {
         };
         insta::assert_snapshot!(
             numeric.to_offset().unwrap_err(),
-            @"due to precision loss, UTC offset '-25:59:59.5' is rounded to a value that is out of bounds: parameter 'offset-seconds' with value 1 is not in the required range of -93599..=93599",
+            @"due to precision loss, offset is rounded to a value that is out of bounds: parameter 'offset-seconds' with value 1 is not in the required range of -93599..=93599",
         );
     }
 }
