@@ -16,6 +16,20 @@ fn conflict(with: &'static str) -> InsertError {
     InsertError::Conflict { with: with.into() }
 }
 
+// Regression test for https://github.com/ibraheemdev/matchit/issues/84.
+#[test]
+fn missing_leading_slash_suffix() {
+    InsertTest(vec![("/{foo}", Ok(())), ("/{foo}suffix", Ok(()))]).run();
+    InsertTest(vec![("{foo}", Ok(())), ("{foo}suffix", Ok(()))]).run();
+}
+
+// Regression test for https://github.com/ibraheemdev/matchit/issues/82.
+#[test]
+fn missing_leading_slash_conflict() {
+    InsertTest(vec![("{foo}/", Ok(())), ("foo/", Ok(()))]).run();
+    InsertTest(vec![("foo/", Ok(())), ("{foo}/", Ok(()))]).run();
+}
+
 #[test]
 fn wildcard_conflict() {
     InsertTest(vec![
