@@ -13,13 +13,13 @@ use alloc::string::{String, ToString};
 use alloc::vec::Vec;
 use core::convert::TryFrom;
 use core::fmt::{self, Display, Formatter, Write};
-#[cfg(feature = "std")]
+#[cfg(not(feature = "no_std"))]
 use core::ops::Mul;
 use core::ops::{Add, Sub};
 use core::str::FromStr;
 use core::time::Duration;
 use core::{iter, mem, str};
-#[cfg(feature = "std")]
+#[cfg(not(feature = "no_std"))]
 use std::time::Instant;
 
 use bitflags::bitflags;
@@ -67,7 +67,7 @@ impl Rgb {
     /// Implementation of [W3C's luminance algorithm].
     ///
     /// [W3C's luminance algorithm]: https://www.w3.org/TR/WCAG20/#relativeluminancedef
-    #[cfg(feature = "std")]
+    #[cfg(not(feature = "no_std"))]
     pub fn luminance(self) -> f64 {
         let channel_luminance = |channel| {
             let channel = channel as f64 / 255.;
@@ -88,7 +88,7 @@ impl Rgb {
     /// Implementation of [W3C's contrast algorithm].
     ///
     /// [W3C's contrast algorithm]: https://www.w3.org/TR/WCAG20/#contrast-ratiodef
-    #[cfg(feature = "std")]
+    #[cfg(not(feature = "no_std"))]
     pub fn contrast(self, other: Rgb) -> f64 {
         let self_luminance = self.luminance();
         let other_luminance = other.luminance();
@@ -104,7 +104,7 @@ impl Rgb {
 }
 
 // A multiply function for Rgb, as the default dim is just *2/3.
-#[cfg(feature = "std")]
+#[cfg(not(feature = "no_std"))]
 impl Mul<f32> for Rgb {
     type Output = Rgb;
 
@@ -266,7 +266,7 @@ impl<T: Timeout> Default for SyncState<T> {
 
 /// The processor wraps a `crate::Parser` to ultimately call methods on a
 /// Handler.
-#[cfg(feature = "std")]
+#[cfg(not(feature = "no_std"))]
 #[derive(Default)]
 pub struct Processor<T: Timeout = StdSyncHandler> {
     state: ProcessorState<T>,
@@ -275,7 +275,7 @@ pub struct Processor<T: Timeout = StdSyncHandler> {
 
 /// The processor wraps a `crate::Parser` to ultimately call methods on a
 /// Handler.
-#[cfg(not(feature = "std"))]
+#[cfg(feature = "no_std")]
 #[derive(Default)]
 pub struct Processor<T: Timeout> {
     state: ProcessorState<T>,
@@ -438,13 +438,13 @@ impl<'a, H: Handler + 'a, T: Timeout> Performer<'a, H, T> {
     }
 }
 
-#[cfg(feature = "std")]
+#[cfg(not(feature = "no_std"))]
 #[derive(Default)]
 pub struct StdSyncHandler {
     timeout: Option<Instant>,
 }
 
-#[cfg(feature = "std")]
+#[cfg(not(feature = "no_std"))]
 impl StdSyncHandler {
     /// Synchronized update expiration time.
     #[inline]
@@ -453,7 +453,7 @@ impl StdSyncHandler {
     }
 }
 
-#[cfg(feature = "std")]
+#[cfg(not(feature = "no_std"))]
 impl Timeout for StdSyncHandler {
     #[inline]
     fn set_timeout(&mut self, duration: Duration) {
@@ -612,9 +612,6 @@ pub trait Handler {
     /// Clear tab stops.
     fn clear_tabs(&mut self, _mode: TabulationClearMode) {}
 
-    /// Set tab stops at every `interval`.
-    fn set_tabs(&mut self, _interval: u16) {}
-
     /// Reset terminal state.
     fn reset_state(&mut self) {}
 
@@ -1577,7 +1574,6 @@ where
             ('E', []) => handler.move_down_and_cr(next_param_or(1) as usize),
             ('F', []) => handler.move_up_and_cr(next_param_or(1) as usize),
             ('G', []) | ('`', []) => handler.goto_col(next_param_or(1) as usize - 1),
-            ('W', [b'?']) if next_param_or(0) == 5 => handler.set_tabs(8),
             ('g', []) => {
                 let mode = match next_param_or(0) {
                     0 => TabulationClearMode::Current,
@@ -2438,7 +2434,7 @@ mod tests {
     }
 
     #[test]
-    #[cfg(feature = "std")]
+    #[cfg(not(feature = "no_std"))]
     fn contrast() {
         let rgb1 = Rgb { r: 0xFF, g: 0xFF, b: 0xFF };
         let rgb2 = Rgb { r: 0x00, g: 0x00, b: 0x00 };
